/*****************************************************************************
 *
 *  kv5500_photo - read a photo from kv5500 like phone.
 *
 * Copyright (c) 2006 Hyo geol, Lee
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 *   3. The name of the author may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

#include <cstdio>
#include <cstring>

#include <string>
#include <sstream>
#include <fstream>

#include <time.h>

#include <usb.h>

#include "kv5500.hpp"
#include "kv5500_photo.hpp"

using namespace KV5500;

bool PhotoInfoScanner::DoScan(struct usb_device* dev)
{
    assert(dev);

    try {

        UsbDev udev(dev);

        /// vendor 1004 and product 6000, and this is not correct test.
        if(DescCheck(dev)) {

            if(!WakeUp(udev)) {

                printf("Cannot wake up phone.\n");

                return false;
            }

            if(!Request(udev))
                return false;
        }

    } catch(std::exception& e) {

        printf("%s\n", e.what());
    }

    return true;
}

bool PhotoInfoScanner::Request(UsbDev& udev)
{
    unsigned int totalPhotoNum = 0;
    if(!GetTotalPhotoNum(udev, totalPhotoNum) || totalPhotoNum > 256)
        return false;

    std::string name;
    unsigned int fileSize = 0;
    char nameReq[6] = { 0xfe, 0x0b, 0x07, 0x00, 0x00, 0x7e };
    char filesizeReq[8] = { 0xfe, 0x0d, 0x07, 0x00, 0x00, 0x00, 0x00, 0x7e };
    for(unsigned int i = 0; i < totalPhotoNum; ++i) {

        nameReq[3] = i + 1;
        if(udev.BulkWrite(nameReq, sizeof(nameReq), 5000) < 0) {

            printf("Got error while send.\n");

            return false;
        }

        if(RecvPhotoName(udev, name) < 0)
            return false;

        filesizeReq[3] = i + 1;
        if(udev.BulkWrite(filesizeReq, sizeof(filesizeReq), 5000) < 0) {

            printf("Got error while send.\n");

            return false;
        }

        if(RecvPhotoFileSize(udev, fileSize) < 0)
            return false;

        photoInfoContainer_.push_back(PhotoInfo(name, fileSize));
    }

    return true;
}

const char* PhotoGetNum::what() const throw()
{
    return "Photo num error.";
}

bool PhotoGetFileScanner::DoScan(struct usb_device* dev)
{
    assert(dev);

    try {

        UsbDev udev(dev);

        /// vendor 1004 and product 6000, and this is not correct test.
        if(DescCheck(dev)) {

            if(!WakeUp(udev)) {

                printf("Cannot wake up phone.\n");

                return false;
            }

            if(!CheckPhotoNum(udev))
                return false;

            if(!PutPhotoFile(udev))
                return false;
        }

    } catch(std::exception& e) {

        printf("%s\n", e.what());
    }

    return true;
}

bool PhotoGetFileScanner::CheckPhotoNum(UsbDev& udev) const
{
    unsigned int totalPhotoNum = 0;
    GetTotalPhotoNum(udev, totalPhotoNum);
    if(totalPhotoNum > 256 || num_ > totalPhotoNum) {

        printf("Total %d photos.\n", totalPhotoNum);

        return false;
    }

    return true;
}

std::string PhotoGetFileScanner::GetFilename() const
{
    std::stringstream ss;

    ss << "kv5500-";

    ss.width(3);
    ss.fill('0');
    ss << num_;

    ss << "-" << time(0) << ".jpg";

    return ss.str();
}

bool PhotoGetFileScanner::PutPhotoFile(UsbDev& udev)
{
    const std::string filename = GetFilename();

    // get filesize, reduce duplicated codes!
    char filesizeReq[8] = { 0xfe, 0x0d, 0x07, 0x00, 0x00, 0x00, 0x00, 0x7e };
    filesizeReq[3] = num_;
    if(udev.BulkWrite(filesizeReq, sizeof(filesizeReq), 5000) < 0) {

        printf("Got error while send.\n");

        return false;
    }

    char buf[3072];
    const int ret = udev.BulkRead(buf, sizeof(buf), 5000);

    /// assume ret must be over 11 bytes.
    if(ret <= 11) {

        printf("Got error while recv.\n");

        return false;
    }

    const unsigned int fileSize = GetRevEndian(buf + 8);

    int leftSize = fileSize;
    std::ofstream os(filename.c_str());
    if(!os) {

        printf("Cannot open file");

        return false;
    }

    SendPrepareTransferPhoto(udev);

    printf("Writing %s .......\n", filename.c_str());

    unsigned int i = 0;
    char fileReq[9] = { 0xfe, 0x00, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x7e };
    fileReq[3] = num_;
    while(leftSize > 0) {

        PrintTransferProcess(fileSize - leftSize, fileSize);

        fileReq[7] = i;
        if(udev.BulkWrite(fileReq, sizeof(fileReq), 5000) < 0) {

            printf("\nGot error while send.\n");

            return false;
        }

        {
            const int ret2 = udev.BulkRead(buf, sizeof(buf), 5000);
            /// assume ret must be over 15 bytes.
            if(ret2 <= 15)
                return false;

            const unsigned int prefix = GetPrefixPos(buf);
            const unsigned int postfix = GetPostfixPos(buf, ret2);

            for(unsigned int j = prefix; (j < ret2 - postfix) && (leftSize > 0); ++j, --leftSize) {
                if(static_cast<unsigned char>(buf[j]) == 0x7d &&
                   static_cast<unsigned char>(buf[j+1]) == 0x5d) {

                    os << static_cast<char>(0x7d);
                    
                    ++j;

                } else if(static_cast<unsigned char>(buf[j]) == 0x7d && 
                          static_cast<unsigned char>(buf[j+1]) == 0x5e) {

                    os << static_cast<char>(0x7e);

                    ++j;

                } else
                    os << buf[j];
            }
        }

        if(leftSize > 0 && i == 256)
            i = 0;
        else
            ++i;
    }

    printf("\n");

    return true;
}

int KV5500::RecvPhotoName(UsbDev& udev, std::string& photoName)
{
    char buf[1000];
    const int ret = udev.BulkRead(buf, sizeof(buf), 5000);

    /// assume ret must be over 37 bytes.
    if(ret <= 37)
        return -1;

    const char* const nameStr = GetString(buf, 26, 11, 26, 0x00);
    photoName = nameStr;
    delete [] nameStr;

    return ret;
}

int KV5500::RecvPhotoFileSize(UsbDev& udev, unsigned int& size)
{
    char buf[1000];
    const int ret = udev.BulkRead(buf, sizeof(buf), 5000);

    /// assume ret must be over 54 bytes.
    if(ret <= 11)
        return -1;

    size = GetRevEndian(buf + 8);

    return ret;
}

bool KV5500::GetTotalPhotoNum(UsbDev& udev, unsigned int& num)
{
    char buf[7] = { 0xfe, 0x0a, 0x07, 0x00, 0x7e };
    if(udev.BulkWrite(buf, sizeof(buf), 5000) < 0) {

        printf("Got error while send.\n");

        return false;
    }

    char rbuf[1000];
    const int ret = udev.BulkRead(rbuf, sizeof(rbuf), 5000);
    if(ret < 8)
        return false;

    num = static_cast<unsigned int>(rbuf[5]);

    return true;
}

bool KV5500::SendPrepareTransferPhoto(UsbDev& udev)
{
    char prepare[5] = { 0xfe, 0x67, 0xe6, 0xfe, 0x7e };
    if(udev.BulkWrite(prepare, sizeof(prepare), 5000) < 0) {

        printf("Got error while send.\n");

        return false;
    }

    udev.BulkRead(0, 1000, 5000);

    return true;
}

unsigned int KV5500::GetPrefixPos(const char* const buf)
{
    unsigned int prefix = 0;
    unsigned int prefixCount = 0;
    while(prefixCount != 12) {

        if(HasSubstStr(buf, prefix)) {

            prefix += 2;

        } else {

            ++prefix;
        }

        ++prefixCount;
    }

    return prefix;
}

void KV5500::GetPhotoInfo(std::vector<PhotoInfo>& vec)
{
    Usb thisUsb;
    PhotoInfoScanner piScanner;

    thisUsb.Scan(piScanner);

    piScanner.GetPhotoInfo(vec);
}

void KV5500::SavePhotoFile(unsigned int num)
{
    Usb thisUsb;
    PhotoGetFileScanner pgScanner(num);

    thisUsb.Scan(pgScanner);
}
